from django.conf import settings
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver

User = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')

class Post(models.Model):
    uuid = models.UUIDField(primary_key=True)
    author = models.CharField(max_length=256)
    slug = models.SlugField(max_length=256, unique=True, null=True)
    heading = models.CharField(max_length=512, null=True)
    content = models.TextField(null=True)
    is_published = models.BooleanField(default=False)
    is_deleted = models.BooleanField(default=False)
    published_on = models.DateTimeField(null=True)
    created_on = models.DateTimeField(auto_now_add=True)

    class Meta:
        get_latest_by = '-published_on'
        ordering = ('-published_on', '-created_on')


class Author(models.Model):
    user = models.OneToOneField(
        User,
        on_delete=models.CASCADE,
        null=True)
    is_author = models.BooleanField(default=False)


@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    """
    Create an author on user creation
    """
    if created:
        Author.objects.create(user=instance)
    instance.author.save()



# Generated by Django 2.0 on 2018-10-22 12:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Author',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_author', models.BooleanField(default=False)),
                ('user', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('uuid', models.UUIDField(primary_key=True, serialize=False)),
                ('author', models.EmailField(max_length=256)),
                ('slug', models.SlugField(max_length=256, null=True, unique=True)),
                ('heading', models.CharField(max_length=512, null=True)),
                ('content', models.TextField(null=True)),
                ('is_published', models.BooleanField(default=False)),
                ('is_deleted', models.BooleanField(default=False)),
                ('published_on', models.DateTimeField(null=True)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-published_on', '-created_on'),
                'get_latest_by': '-published_on',
            },
        ),
    ]



import uuid

from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import PermissionDenied
from django.core.paginator import Paginator
from django.shortcuts import render, redirect, get_object_or_404
from django.views import View
from django.http import HttpResponse, JsonResponse
from django.utils.text import slugify
from django.utils import timezone

from .models import Post
from .mixins import AuthorRequiredMixin
from .settings import settings as app_settings

common_context = {
    'settings': app_settings
}


class BlogHomeView(View):
    
    def get(self, request):
        posts_list = Post.objects.filter(is_deleted=False, is_published=True)
        paginator = Paginator(posts_list, 15)

        page = request.GET.get('page')
        posts = paginator.get_page(page)
        return render(request, 'django-blogs/home.html', {
            'posts': posts,
            **common_context,
        })

class NewBlogView(LoginRequiredMixin, AuthorRequiredMixin, View):

    def get(self, request):
        post = Post.objects.create(
            author=request.user.username,
            uuid=uuid.uuid4(),
            heading="Click to edit Title",
            content="<p>Click to edit body</p>"
        )
        return redirect('blogs:edit', uuid=post.uuid)

class BlogEditView(LoginRequiredMixin, AuthorRequiredMixin, View):

    def get(self, request, uuid):
        post = get_object_or_404(Post, pk=uuid)
        if not post.author == request.user.username:
            raise PermissionDenied
        return render(request, 'django-blogs/editor.html', {
            'blog': post,
            **common_context,
        })
    
    def post(self, request, uuid):
        if not all([key in request.POST for key in ['heading', 'content']]):
            return JsonResponse(
                {
                    'error': "'heading' and 'content' required"
                },
                status=400)
        post = get_object_or_404(Post, pk=uuid)
        if not post.author == request.user.username:
            raise PermissionDenied
        post.heading = request.POST['heading']
        post.slug = '-'.join([
            slugify(request.POST['heading'])[:200],
            post.uuid.hex[:12]
        ])
        post.content = request.POST['content']
        post.save()
        return HttpResponse(status=200)

class BlogDraftView(LoginRequiredMixin, AuthorRequiredMixin, View):

    def get(self, request):
        posts_list = Post.objects.filter(
            author__exact=request.user.username,
            is_deleted=False,
            is_published=False)
        paginator = Paginator(posts_list, 15)

        page = request.GET.get('page')
        posts = paginator.get_page(page)
        return render(request, 'django-blogs/drafts.html', {
            'posts': posts,
            **common_context,
        })

class BlogReadView(View):

    def get(self, request, slug):
        post = get_object_or_404(
            Post,
            slug=slug,
            is_deleted=False,
            is_published=True)
        return render(request, 'django-blogs/blog.html', {
            'post': post,
            **common_context
        })

class BlogPublishView(LoginRequiredMixin, AuthorRequiredMixin, View):

    def get(self, request, uuid):
        post = get_object_or_404(Post, pk=uuid)
        if not post.author == request.user.username:
            raise PermissionDenied
        post.is_published = True
        post.published_on = timezone.now()
        post.slug = '-'.join([
            slugify(post.heading)[:200],
            post.uuid.hex[:12]
        ])
        post.save()
        return redirect("blogs:read", slug=post.slug)

class BlogDeleteView(LoginRequiredMixin, AuthorRequiredMixin, View):

    def get(self, request, uuid):
        post = get_object_or_404(Post, pk=uuid)
        if not post.author == request.user.username:
            raise PermissionDenied
        post.is_deleted = True
        post.save()
        return redirect("blogs:home")


from django.conf import settings as project_settings

class BlogSettings(object):
    def __init__(self):
        pass

settings = BlogSettings()

settings.SHOW_FOOTER_LINK = getattr(project_settings, 'BLOGS_SHOW_FOOTER_LINK', True)
settings.PARENT_SITE_NAME = getattr(project_settings, 'BLOGS_PARENT_SITE_NAME', 'Email Gardener')


from django.contrib import admin

from blogs.models import Post, Author

@admin.register(Post)
class PostAdmin(admin.ModelAdmin):

    list_display = ('uuid', 'heading', 'is_published')
    list_filter = ('is_published', 'is_deleted')

@admin.register(Author)
class AuthorAdmin(admin.ModelAdmin):

    list_display = ('user', 'is_author')
    list_filter = ('is_author', )


from setuptools import setup, find_packages
from blogs import __version__

def readfile(filename):
    return open(filename, 'r').read()

print(__version__)
long_description = readfile('README.md')

setup(
    version=__version__,
    name='djangoblogs',
    description='Django blogging with medium style editor',
    long_description=long_description,
    url='https://github.com/arjunsinghy96/django-blogs',
    author='Arjun Singh Yadav',
    author_email='arjunsinghy96@gmail.com',
    license='MIT',
    keywords='django blogs medium blog',
    include_package_data=True,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        ],
    packages=find_packages(exclude=['blogs.tests', 'config']),
    install_requires=[
        'django>=2.0'
    ]
    )



from django.urls import path

from . import views

app_name = 'blogs'

urlpatterns = [
    path('', views.BlogHomeView.as_view(), name="home"),
    path('new/', views.NewBlogView.as_view(), name="new"),
    path('<uuid:uuid>/edit/', views.BlogEditView.as_view(), name="edit"),
    path('drafts/', views.BlogDraftView.as_view(), name="drafts"),
    path('<slug:slug>/', views.BlogReadView.as_view(), name="read"),
    path('<uuid:uuid>/publish/', views.BlogPublishView.as_view(), name="publish"),
    path('<uuid:uuid>/delete/', views.BlogDeleteView.as_view(), name="delete"),
]


from django.core.exceptions import PermissionDenied
from .models import Author

class AuthorRequiredMixin:
    """
    Checks if the current user is an author
    """
    def dispatch(self, request, *args, **kwargs):
        try:
            author = Author.objects.get(user=request.user)
            if author.is_author:
                return super().dispatch(request, *args, **kwargs)
            else:
                raise PermissionDenied
        except Author.DoesNotExist: # pragma: no cover
            raise PermissionDenied        